/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2003 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  Main DLL routine
 *--------------------------------------------------------------------------
 * @histroy: 16.09.2003. Created
 */
#include "Spline.h"
#include "SplineNode.h"
#include "SplineTechnique.h"
#include "CreateSpline.h"
#include "ControlPointType.h"
#include "ModifyTools.h"
#include "controlPointProperties.h" 
#include <helpers/services.h>

ZM_LIBRARY_DEFAULT_DECLARE()
  ZM_LIBRARY_NO_COPYRIGHT_INFO()
  ZM_LIBRARY_COLLECTION_BEGIN()
    ZM_LIBRARY_REG(scene::CSpline,                    ZM_CLASS_SHARED)
    ZM_LIBRARY_REG(scene::CSplineNode,                ZM_CLASS_SHARED)
    ZM_LIBRARY_REG(scene::CControlPoint,              ZM_CLASS_SHARED)
    ZM_LIBRARY_REG(rend::CSplineRenderTechnique,      ZM_CLASS_SHARED)
    ZM_LIBRARY_REG(core::tools::CStripSplineTool,     ZM_CLASS_TOOL)
    ZM_LIBRARY_REG(core::tools::CCircleSplineTool,    ZM_CLASS_TOOL)
    ZM_LIBRARY_REG(core::tools::CHelixSplineTool,     ZM_CLASS_TOOL)

    ZM_LIBRARY_REG(core::tools::CControlPointType,    ZM_CLASS_TOOL)
    ZM_LIBRARY_REG(core::tools::CMakePointFirstTool,  ZM_CLASS_TOOL)
  ZM_LIBRARY_COLLECTION_END()
  ZM_LIBRARY_REQUIRES(ZM_LIBREQUIRES_INIT)
  ZM_LIBRARY_INITIALIZATION(initLibrary)
ZM_LIBRARY_DEFAULT_END()


ZM_DLLMAIN_DEFAULT_BEGIN()
  ZM_LIBRARY_DEFAULT()
  ZM_LIBRARY_DESCRIPTION(_T("ZModeler 2 Splines library"))
  ZM_LIBRARY_REGISTER()
ZM_DLLMAIN_DEFAULT_END()

ZRESULT initLibrary(void)
{
  //
  // register in node service
  //
  ZPtr<scene::IScene> pScene;
  g_pZModeler->getScene(&pScene);
  ZPtr<services::INodeService> pNSrv;
  pScene->getNodeService(&pNSrv);
  if (pNSrv)
    pNSrv->installNodeClass(_T("scene::CSplineNode"), IID_(scene::ISplineNode));
  //
  // add render technique compatibility:
  //
  ZPtr<core::ICompatibilityInfo> pComp;
  g_pZModeler->getCurrentCompatibility(&pComp);
  if (pComp)
    pComp->setRenderTechnique(IID_(scene::ISplineNode), _T("rend::CSplineRenderTechnique"));
  //
  // register properties source
  //
  ZPtr<core::tools::CControlPointProperties> pProps;
  core::tools::CControlPointProperties::CreateInstance(&pProps);
  ZPtr<services::IPropertiesService> pSrv;
  if (ZRESULT_OK == services::getPropertiesService(&pSrv))
    pSrv->addPropertiesSource(pProps);
  pSrv = NULL;

  //
  // register global configuration
  //
  if (ZRESULT_OK == services::getStaticPropertiesConfiguration(&pSrv))
  {
    ZStringSeq seq;
    seq.add(_T("Views"));
    seq.add(_T("Splines"));
    pSrv->addBoolProperty(seq, _T("Use autodetalization"), &scene::g_bAutoDetail);
    pSrv->addLongRangeProperty(seq, _T("Default Detalization"), &scene::g_nDetail, 0, 1000);
    pSrv->addLongRangeProperty(seq, _T("Maximum Detalization"), &scene::g_nMaxDetail, 1, 4000);
    pSrv->addFloatRangeProperty(seq, _T("Autodetalization Precision"), &scene::g_fPrecision, 0.0f, 1.0f);
  }
  return ZRESULT_OK;
}

