/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2003 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x Tools that change head control point
 *--------------------------------------------------------------------------
 * @histroy: 02.10.2003. Created
 */
#include "Spline.h"
#include "SplineNode.h"
#include "ModifyTools.h"
#include <helpers/undo.h>

namespace core
{
namespace tools
{

INTERFACE_BEGINMAP(CMakePointFirstTool)
  INTERFACE_CHAIN(CContextMenuToolEx)
INTERFACE_ENDMAP()

CMakePointFirstTool::CMakePointFirstTool() : CContextMenuToolEx()
{
  m_strName = m_strTitle = _T("Make First");
  m_seqBranch.add(_T("Edit"));
}

CMakePointFirstTool::~CMakePointFirstTool()
{
}


//-------------------------------------------------------
// @name : CMakePointFirstTool::queryLayout
// Queries whether tool can be laid in context menu
// @param scene::INode* pNode            : 
// @param core::eLevel level             : 
// @param core::ui::IViewport* pViewport : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMakePointFirstTool::queryLayout(scene::INode* pNode, core::eLevel level, core::ui::IViewport* pViewport)
{
  if (NULL == pNode || level != core::LEV_MANIPULATOR)
    return ZRESULT_FALSE;
  ZPtr<scene::ISplineNode> pSplineNode;
  if (ZRESULT_OK != pNode->QueryInterface(IID_(scene::ISplineNode), (void**)&pSplineNode))
    return ZRESULT_FALSE;
  if (ZRESULT_OK != pSplineNode->getSpline(&m_pSpline))
    return ZRESULT_FALSE;
  m_pSpline->Release();//StdPtr
  //
  // navigate to active control point
  //
  scene::ISpline::lpNode ptr = m_pSpline->head();
  m_pPoint = NULL;
  m_pNode = pSplineNode;
  if (NULL == ptr)
    return ZRESULT_FALSE;
  while (ptr)
  {
    if (ptr->m_pIntr->checkStatus(STATUS_ACTIVE))
    {
      if (ptr == m_pSpline->head())
        return ZRESULT_FALSE;//can't make this point first
      if (ptr->m_pNext && m_pSpline->isClosed() != ZRESULT_OK)
        return ZRESULT_FALSE;//can't make this point first
      m_pPoint = ptr->m_pIntr;
      break;
    }
    ptr = ptr->m_pNext;
  };
  return m_pPoint ? ZRESULT_OK : ZRESULT_FALSE;
}



//-------------------------------------------------------
// @name : CMakePointFirstTool::apply
// Change type
// @param core::IProcParams* pParams     : 
// @param DWORD* pRetVal                 : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMakePointFirstTool::apply(core::IProcParams* pParams, DWORD* pRetVal)
{
  if (!m_pNode || !m_pSpline || !m_pPoint)
    return ZRESULT_FALSE;
  //undo:
  ZPtr<scene::ISplineNode> pSPNode;
  m_pNode->QueryInterface(IID_(scene::ISplineNode), (void**)&pSPNode);
  core::undo::undoSplineOperation(_T("Reorder spline"), 
                                  core::undo::UNDO_SPLINE_GEOMETRY,
                                  pParams,
                                  pSPNode);
  //reorder collection
  m_pSpline->pushState();
  m_pSpline->reset();
  if (ZRESULT_OK != m_pSpline->isClosed())
    m_pSpline->reverseOrder();
  else
  {//closed spline
    scene::ISpline::lpNode ptrHead = m_pSpline->head();
    scene::ISpline::lpNode ptr = ptrHead;
    scene::ISpline::lpNode newHead = NULL;
    while (ptr)
      if (ptr->m_pNext->m_pIntr == m_pPoint)
      {
        newHead = ptr->m_pNext;
        ptr->m_pNext = NULL;
        ptr = newHead;
        while (ptr->m_pNext)
          ptr = ptr->m_pNext;
        ptr->m_pNext = ptrHead;
        break;
      }
      else
        ptr = ptr->m_pNext;
    m_pSpline->head() = newHead;
  }//closed spline
  m_pNode->changed(NC_VISUAL | NC_CHILD | NC_GEOMETRY);
  if (pRetVal)
    *pRetVal |= ZRESULT_REDRAW_ALL;
  return ZRESULT_OK;
}


};//tools
};//core



