/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                   Copyright (c) 2002 Zanoza Software.                   *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x Splines render technique implementation
 *--------------------------------------------------------------------------
 * @histroy: 17.09.2003. Created
 */
#include "SplineTechnique.h"
#include <scene/IManipulateable.h>
#include <scene/ISplineNode.h>

namespace rend
{

INTERFACE_BEGINMAP(CSplineRenderTechnique)
  INTERFACE_ENTRY(rend::IRenderTechnique)
  INTERFACE_ENTRY(core::INamed)
  INTERFACE_ENTRY(core::io::ISerializeable)
  INTERFACE_ENTRY(core::IErrorInfo)
INTERFACE_ENDMAP()

//
// static datamembers
//
ZStdPtr<core::ui::IGraphManager> CSplineRenderTechnique::m_pGraphManager;
std::map<core::ui::eColorResource, DWORD> CSplineRenderTechnique::g_sMapCol;


CSplineRenderTechnique::CSplineRenderTechnique() : core::CErrorInfo()
{
  m_strName = m_strTitle = _T("Splines Render Technique");
  m_nVertexBufferCount    = 0;
  m_nVertexBufferSize     = 0;
  m_dwVertexStride        = 16;//x,y,z,color

  if (!m_pGraphManager)
  {
    //
    // pickup graph manager
    //
    ZPtr<core::ui::IUIManager>        pUIManager;
    g_pZModeler->getUIManager(&pUIManager);
    if (pUIManager)
      pUIManager->getGraphManager(&m_pGraphManager);
    if (m_pGraphManager)
      m_pGraphManager->Release(); //it's StdPtr
    //init colors
    ZColor col;
    m_pGraphManager->getColor(core::ui::ZCOLOR_CURVE_DEFAULT, col);
    g_sMapCol[core::ui::ZCOLOR_CURVE_DEFAULT] = col.getARGB();
    m_pGraphManager->getColor(core::ui::ZCOLOR_CURVE_DISABLED, col);
    g_sMapCol[core::ui::ZCOLOR_CURVE_DISABLED] = col.getARGB();
    m_pGraphManager->getColor(core::ui::ZCOLOR_CURVE_ACTIVE, col);
    g_sMapCol[core::ui::ZCOLOR_CURVE_ACTIVE] = col.getARGB();
    m_pGraphManager->getColor(core::ui::ZCOLOR_CURVE_SELECTED, col);
    g_sMapCol[core::ui::ZCOLOR_CURVE_SELECTED] = col.getARGB();
    m_pGraphManager->getColor(core::ui::ZCOLOR_CURVE_SELACTIVE, col);
    g_sMapCol[core::ui::ZCOLOR_CURVE_SELACTIVE] = col.getARGB();
    //
    m_pGraphManager->getColor(core::ui::ZCOLOR_MANIP_DEFAULT, col);
    g_sMapCol[core::ui::ZCOLOR_MANIP_DEFAULT] = col.getARGB();
    m_pGraphManager->getColor(core::ui::ZCOLOR_MANIP_DISABLED, col);
    g_sMapCol[core::ui::ZCOLOR_MANIP_DISABLED] = col.getARGB();
    m_pGraphManager->getColor(core::ui::ZCOLOR_MANIP_ACTIVE, col);
    g_sMapCol[core::ui::ZCOLOR_MANIP_ACTIVE] = col.getARGB();
    m_pGraphManager->getColor(core::ui::ZCOLOR_MANIP_SELECTED, col);
    g_sMapCol[core::ui::ZCOLOR_MANIP_SELECTED] = col.getARGB();
    m_pGraphManager->getColor(core::ui::ZCOLOR_MANIP_SELACTIVE, col);
    g_sMapCol[core::ui::ZCOLOR_MANIP_SELACTIVE] = col.getARGB();

  }
}


CSplineRenderTechnique::~CSplineRenderTechnique()
{
}

//-----------------------
// rend::IRenderTechnique
//-----------------------

ZRESULT CSplineRenderTechnique::validate(IDevice* pDevice, scene::INode* pNode)
{
  ZPtr<scene::ISplineNode>  pSplineNode;
  return pNode->QueryInterface(IID_(scene::ISplineNode), (void**)&pSplineNode);
}


//-----------------------
// core::INamed
//-----------------------
ZRESULT CSplineRenderTechnique::getName(ZString& str)
{
  str = m_strName;
  return ZRESULT_OK;
}

ZRESULT CSplineRenderTechnique::setName(ZString str)
{
  m_strName = str;
  return ZRESULT_OK;
}

ZRESULT CSplineRenderTechnique::getTitle(ZString& str)
{
  str = m_strTitle;
  return ZRESULT_OK;
}

ZRESULT CSplineRenderTechnique::setTitle(ZString str)
{
  m_strTitle = str;
  return ZRESULT_OK;
}

ZRESULT CSplineRenderTechnique::getDescription(ZStringSeq& seqDesc)
{
  seqDesc = m_seqDesc;
  return ZRESULT_OK;
}

ZRESULT CSplineRenderTechnique::setDescription(ZStringSeq& seqDesc)
{
  m_seqDesc = seqDesc;
  return ZRESULT_OK;
}

//-------------------------------- rendering technique main -------------------------

//-------------------------------------------------------
// @name : CSplineRenderTechnique::onNodeChange
// Node validation routine
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineRenderTechnique::nodeChange(scene::INode* pNode, DWORD dwBits)
{
  if (!pNode)
    return ZRESULT_OK;
  if (!m_pNode)
  {
    pNode->QueryInterface(IID_(scene::ISplineNode), (void**)&m_pNode);
    if (!m_pNode)
      return ZRESULT_OK;
    else
      m_pNode->Release();//std ptr
  }

  m_seqBlocks.length(0);
  m_nVertexBufferCount = 0;

  ZPtr<scene::ISpline>  pSpline;
  m_pNode->getSpline(&pSpline);
  if (!pSpline)
    return ZRESULT_FAIL;
  ZPtr<IDevice>     pDevice;
  g_pZModeler->getDefaultDevice(&pDevice);
  if (!pDevice)
    return ZRESULT_FAIL;
  //
  // determin edit-level:
  //
  ZPtr<core::IProcParams> pParams;
  g_pZModeler->getProcParams(&pParams);
  if (pParams)
    pParams->getLevel(m_level);
  else
    m_level = core::LEV_NODE;
  //
  // determine node status:
  //
  m_dwStatus = m_pNode->getStatus();
  if (m_dwStatus & STATUS_HIDDEN)
    return ZRESULT_OK;


  pSpline->pushState();
  resizeBuffer(pDevice, pSpline);  //resize buffer if required;
  pSpline->popState();
  if (!m_pVertices)
    return ZRESULT_FALSE;//can't render;

  pSpline->pushState();
  BYTE* pData = NULL;
  m_pVertices->Lock(0, 0, (void**)&pData, D3DLOCK_DISCARD);
  initSplineLines(pSpline, pData);
  initSplineManipulators(pSpline, pData); //vector lines
  initSplineDots(pSpline, pData);
  pSpline->popState();

  m_pVertices->Unlock();

  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CSplineRenderTechnique::renderNode
// Main routine that renders the node itself.
// @param scene::INode* pNode            : 
// @param core::ui::IViewport* pViewport : 
// @param IDevice* pDevice               : 
// @param ZMatrixSeq& matrixStack        : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineRenderTechnique::renderNode(scene::INode* pNode, core::ui::IViewport* pViewport, IDevice* pDevice, ZMatrixSeq& matrixStack)
{
  resetError();
  if ((NULL == pNode) || (NULL == pViewport))
    return ZRESULT_INVALID_ARG;
  if (!m_pNode)
    nodeChange(pNode, NC_VISUAL | NC_GEOMETRY);

  //
  // go render!
  //
  ZPtr<core::IProcParams>         pParams;
  ZPtr<services::IRenderManager>  pRendMan;
  if ((ZRESULT_OK == g_pZModeler->getProcParams(&pParams)) &&
      pParams &&
      (ZRESULT_OK == pParams->getRenderManager(&pRendMan)) &&
      pRendMan)
    // RENDER!
    pRendMan->renderBlocks(m_seqBlocks);
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CSplineRenderTechnique::resizeBuffer
// Resizes vertex buffer if required
// @param Device* pDevice                : 
// @param ISpline* pSpline               : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineRenderTechnique::resizeBuffer(IDevice* pDevice, scene::ISpline* pSpline)
{
  m_nVertexBufferCount = 0;
  long nNewSize = 0;
  if (NULL == pDevice || NULL == pSpline)
    return ZRESULT_INVALID_ARG;
  nNewSize = pSpline->numCurvePoints(false)+1;//1 is big head vertex
  long numControls = pSpline->numCurvePoints(true);
  nNewSize += numControls*(1+4+2);//vertex, 2 lines, 2 dots.

  if (nNewSize*m_dwVertexStride > (DWORD)m_nVertexBufferSize)
  {
    //realloaction
    m_nVertexBufferSize = nNewSize*m_dwVertexStride;
    m_pVertices = NULL; //release
    pDevice->CreateVertexBuffer(m_nVertexBufferSize,
                                D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC,
                                D3DFVF_XYZ | D3DFVF_DIFFUSE,
                                D3DPOOL_DEFAULT,
                                &m_pVertices,
                                NULL);
    m_seqBlocks.clear();
  }
  return ZRESULT_OK;
}


// @name : CSplineRenderTechnique::initSplineLines
// Inits spline lines
// @param VertexBuf* pVertBuf            : 
// @param ZPolyBuf& polyBuf              : 
// @param BYTE* pBuffer                 : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineRenderTechnique::initSplineLines(scene::ISpline* pSpline, BYTE* pBuffer)
{
  if (NULL == pSpline || NULL == pBuffer)
    return ZRESULT_FALSE;
  pSpline->reset();
  ZPtr<scene::IControlPoint>  pStart, pEnd;
  pSpline->next(&pStart);
  if (!pStart)
    return ZRESULT_FALSE;

  tRenderBlock* pBlock = &m_seqBlocks[m_seqBlocks.length()];
  pBlock->m_type           = rend::LINESTRIP;
  pBlock->m_nMaterial      = -1;
  pBlock->m_pIndexes       = NULL;
  pBlock->m_pVertices      = m_pVertices;
  pBlock->m_nFirstIndex    = m_nVertexBufferCount;
  pBlock->m_nPrimitiveCount= 0;//++
  pBlock->m_nVertices      = 0;//++
  pBlock->m_dwVertexStride = m_dwVertexStride;
  pBlock->m_dwFlags        = RENDERBLOCK_ZTEST  |
                             RENDERBLOCK_ZWRITE |
                             RENDERBLOCK_DOUBLESIDED |
                             RENDERBLOCK_WIREFRAME |
                             RENDERBLOCK_USECOLOR;
  core::ui::eColorResource res;
  if (0==(m_dwStatus & STATUS_ENABLED))
    res = core::ui::ZCOLOR_CURVE_DISABLED;
  else
    if (m_dwStatus & STATUS_SELECTED)
      res = (m_dwStatus & STATUS_ACTIVE) ? core::ui::ZCOLOR_CURVE_SELACTIVE : core::ui::ZCOLOR_CURVE_SELECTED;
    else
      res = (m_dwStatus & STATUS_ACTIVE) ? core::ui::ZCOLOR_CURVE_ACTIVE : core::ui::ZCOLOR_CURVE_DEFAULT;
  pBlock->m_color = g_sMapCol[res];

  pBlock->m_dwFVF          = D3DFVF_XYZ | D3DFVF_DIFFUSE;

  ZRESULT zr;
  do
  {
    td3dvertex vertex;
    ZPoint3D pt;
    vertex.dwColor = pBlock->m_color.getARGB();

    zr = pSpline->getSectionControls(pStart, &pEnd);
    if (pStart && pEnd)
    {
      //
      // valid section
      //
      long nDetail;
      pStart->getDetalization(nDetail);
      for (int i = 0; i < nDetail+1; i++)
      {
        //get point
        pSpline->getSectionPoint(pStart, pEnd, (float)i/(float)(nDetail+1), pt);
        vertex = pt;
        memcpy((pBuffer + m_nVertexBufferCount*m_dwVertexStride), &vertex, m_dwVertexStride);
        m_nVertexBufferCount++;
        pBlock->m_nPrimitiveCount++;
        pBlock->m_nVertices++;
      }
    }
    if (ZRESULT_FALSE == zr && pEnd)
    {
      //write last point
      pEnd->getControlPosition(pt);
      vertex = pt;
      memcpy((pBuffer + m_nVertexBufferCount*m_dwVertexStride), &vertex, m_dwVertexStride);
      m_nVertexBufferCount++;
      pBlock->m_nVertices++;
    }
    pStart = pEnd;
    pEnd   = NULL;
  }while(ZRESULT_OK == zr);
  
  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CSplineRenderTechnique::initSplineDots
// Initializes spline dots
// @param scene::ISpline* pSpline        : 
// @param BYTE* pBuffer                  : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineRenderTechnique::initSplineDots(scene::ISpline* pSpline, BYTE* pBuffer)
{
  if (NULL == pSpline || NULL == pBuffer)
    return ZRESULT_FALSE;
  //
  // head point:
  //
  ZPtr<scene::IControlPoint>  pStart;
  ZPoint3D pt;
  pSpline->reset();
  pSpline->next(&pStart);
  if (!pStart)
    return ZRESULT_FALSE;

  tRenderBlock* pBlock = &m_seqBlocks[m_seqBlocks.length()];
  pBlock->m_type           = rend::BIGPOINTS;
  pBlock->m_nMaterial      = -1;
  pBlock->m_pIndexes       = NULL;
  pBlock->m_pVertices      = m_pVertices;
  pBlock->m_nFirstIndex    = m_nVertexBufferCount;
  pBlock->m_nPrimitiveCount= 0;//++
  pBlock->m_nVertices      = 0;//++
  pBlock->m_dwVertexStride = m_dwVertexStride;
  pBlock->m_dwFlags        = RENDERBLOCK_ZTEST  |
                             RENDERBLOCK_ZWRITE |
                             RENDERBLOCK_DOUBLESIDED |
                             RENDERBLOCK_WIREFRAME;
  //color
  pBlock->m_dwFVF          = D3DFVF_XYZ | D3DFVF_DIFFUSE;
  core::ui::eColorResource res;
  if (SUBLEVEL(m_level))
  {
    DWORD dwStatus = pStart->getStatus();
    if (0==(dwStatus & STATUS_ENABLED))
      res = core::ui::ZCOLOR_CURVE_DISABLED;
    else
      if (dwStatus & STATUS_SELECTED)
        res = (dwStatus & STATUS_ACTIVE) ? core::ui::ZCOLOR_CURVE_SELACTIVE : core::ui::ZCOLOR_CURVE_SELECTED;
      else
        res = (dwStatus & STATUS_ACTIVE) ? core::ui::ZCOLOR_CURVE_ACTIVE : core::ui::ZCOLOR_CURVE_DEFAULT;
  }
  else
    res = (m_dwStatus & STATUS_ACTIVE) ? core::ui::ZCOLOR_CURVE_ACTIVE : core::ui::ZCOLOR_CURVE_DEFAULT;
  td3dvertex vertex;
  pStart->getControlPosition(pt);
  vertex = pt;
  vertex.dwColor = g_sMapCol[res];
  memcpy((pBuffer + m_nVertexBufferCount*m_dwVertexStride), &vertex, m_dwVertexStride);
  m_nVertexBufferCount++;
  pBlock->m_nPrimitiveCount++;
  pBlock->m_nVertices++;

  //
  // other points
  //
  if (core::LEV_MANIPULATOR != m_level || 0==(m_dwStatus & STATUS_ACTIVE))
    return ZRESULT_OK;

  pBlock = &m_seqBlocks[m_seqBlocks.length()];
  pBlock->m_type           = rend::MIDPOINTS;
  pBlock->m_nMaterial      = -1;
  pBlock->m_pIndexes       = NULL;
  pBlock->m_pVertices      = m_pVertices;
  pBlock->m_nFirstIndex    = m_nVertexBufferCount;
  pBlock->m_nPrimitiveCount= 0;//++
  pBlock->m_nVertices      = 0;//++
  pBlock->m_dwVertexStride = m_dwVertexStride;
  pBlock->m_dwFlags        = RENDERBLOCK_ZTEST  |
                             RENDERBLOCK_ZWRITE |
                             RENDERBLOCK_DOUBLESIDED  |
                             RENDERBLOCK_WIREFRAME;
  //color
  pBlock->m_dwFVF          = D3DFVF_XYZ | D3DFVF_DIFFUSE;

  ZPtr<scene::IManipulateable>  pManipable;
  ZPtr<scene::CManipulatorCol>  pManips;
  ZPtr<scene::IManipulator>     pManip;
  if (ZRESULT_OK != m_pNode->QueryInterface(IID_(scene::IManipulateable), (void**)&pManipable))
    return ZRESULT_OK;
  pManipable->getManipulators(&pManips);
  if (!pManips)
    return ZRESULT_OK;
  pManips->pushState();
  pManips->reset();
  ZMatrix mat;
  while (ZRESULT_OK == pManips->next(&pManip))
  {
    DWORD dwStatus = pManip->getStatus();
    if (0==(dwStatus & STATUS_HIDDEN))
    {
      pManip->getTransform(mat);
      mat.getTranslation(pt);
      vertex = pt;
      if (0==(dwStatus & STATUS_ENABLED))
        res = core::ui::ZCOLOR_MANIP_DISABLED;
      else
        if (dwStatus & STATUS_SELECTED)
          res = (dwStatus & STATUS_ACTIVE) ? core::ui::ZCOLOR_MANIP_SELACTIVE : core::ui::ZCOLOR_MANIP_SELECTED;
        else
          res = (dwStatus & STATUS_ACTIVE) ? core::ui::ZCOLOR_MANIP_ACTIVE : core::ui::ZCOLOR_MANIP_DEFAULT;
      vertex.dwColor = g_sMapCol[res];
      memcpy((pBuffer + m_nVertexBufferCount*m_dwVertexStride), &vertex, m_dwVertexStride);
      m_nVertexBufferCount++;
      pBlock->m_nPrimitiveCount++;
      pBlock->m_nVertices++;
    }
    pManip = NULL;
  }
  pManips->popState();
  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CSplineRenderTechnique::initSplineManipulators
// Initializes spline manipulators
// @param scene::ISpline* pSpline        : 
// @param BYTE* pBuffer                  : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineRenderTechnique::initSplineManipulators(scene::ISpline* pSpline, BYTE* pBuffer)
{
  if (NULL == pSpline || NULL == pBuffer || core::LEV_MANIPULATOR != m_level ||
      0==(m_dwStatus & STATUS_ACTIVE))
    return ZRESULT_FALSE;

  tRenderBlock* pBlock = &m_seqBlocks[m_seqBlocks.length()];
  pBlock->m_type           = rend::LINES;
  pBlock->m_nMaterial      = -1;
  pBlock->m_pIndexes       = NULL;
  pBlock->m_pVertices      = m_pVertices;
  pBlock->m_nFirstIndex    = m_nVertexBufferCount;
  pBlock->m_nPrimitiveCount= 0;//++
  pBlock->m_nVertices      = 0;//++
  pBlock->m_dwVertexStride = m_dwVertexStride;
  pBlock->m_dwFlags        = RENDERBLOCK_ZTEST  |
                             RENDERBLOCK_ZWRITE |
                             RENDERBLOCK_DOUBLESIDED  |
                             RENDERBLOCK_WIREFRAME |
                             RENDERBLOCK_USECOLOR;
  pBlock->m_dwFVF          = D3DFVF_XYZ | D3DFVF_DIFFUSE;
  if (m_pGraphManager)
    m_pGraphManager->getColor(core::ui::ZCOLOR_MESH_NORMALS, pBlock->m_color);
  else
    pBlock->m_color = 0xFF808080;

  ZPtr<scene::IControlPoint>  pStart;
  pSpline->reset();
  scene::eControlPoint type;
  td3dvertex pt, p;
  ZPoint3D _p;
  pt.dwColor = p.dwColor = pBlock->m_color.getARGB();
  ZVector  v;
  bool bFirst = true;
  while (ZRESULT_OK == pSpline->next(&pStart))
  {
    pStart->getControlPosition(_p);
    pt = _p;
    pStart->getType(type);
    if (type != scene::SCP_CORNER)
    {
      if (ZRESULT_OK == pSpline->isClosed() || !bFirst)
      {//has outgoing
        pStart->getControlVector(scene::SCV_U1, v);
        memcpy(pBuffer + m_nVertexBufferCount*m_dwVertexStride, &pt, m_dwVertexStride);
        m_nVertexBufferCount++;
        pBlock->m_nVertices++;
        p.x = pt.x + v.x;
        p.y = pt.y + v.y;
        p.z = pt.z + v.z;
        memcpy(pBuffer + m_nVertexBufferCount*m_dwVertexStride, &p, m_dwVertexStride);
        m_nVertexBufferCount++;
        pBlock->m_nVertices++;

        pBlock->m_nPrimitiveCount++;
      }//has outgoing
      if (ZRESULT_OK == pSpline->isClosed() || NULL != pSpline->current())
      {//has incoming
        pStart->getControlVector(scene::SCV_U0, v);
        memcpy(pBuffer + m_nVertexBufferCount*m_dwVertexStride, &pt, m_dwVertexStride);
        m_nVertexBufferCount++;
        pBlock->m_nVertices++;
        p.x = pt.x + v.x;
        p.y = pt.y + v.y;
        p.z = pt.z + v.z;
        memcpy(pBuffer + m_nVertexBufferCount*m_dwVertexStride, &p, m_dwVertexStride);
        m_nVertexBufferCount++;
        pBlock->m_nVertices++;

        pBlock->m_nPrimitiveCount++;
      }//has incoming
    }
    bFirst = false;
    pStart = NULL;
  }
  return ZRESULT_OK;
}

};//rend

