/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                   Copyright (c) 2002 Zanoza Software.                   *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x CSplineNode serialization implementation
 *--------------------------------------------------------------------------
 * @histroy: 05.01.2004. Created
 */
#include "Spline.h"
#include "SplineNode.h"
#include "SplineTechnique.h"

namespace scene
{


//-------------------------------------------------------
// @name : CSplineNode::saveDeclaration
// Saves declaration
// @param core::io::IStream* pStream     : 
// @param services::IOpenSaveService* pSrv : 
// @param LPCTSTR pszResolver            : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineNode::saveDeclaration(core::io::IStream* pStream, services::IOpenSaveService* pSrv, LPCTSTR pszResolver)
{
  //
  // note: I save render technique as a "sheadule saving of this interface" (registerSerializeable),
  // while spline is saved explicitly before node declaration (direct "saveDeclaration" call);
  // this is because I NEED spline data be saved (and recreated) before spline node itself
  // is saved (and loaded).
  core::io::registerSerializeable(m_pRenderTechnique->GetControllingUnknown(), false);
  ZPtr<core::io::ISerializeable>  pSerial;
  if (ZRESULT_OK == m_pSpline->QueryInterface(IID_(core::io::ISerializeable), (void**)&pSerial))
    pSerial->saveDeclaration(pStream, pSrv, NULL);
  //core::io::registerSerializeable(m_pSpline->GetControllingUnknown(), false);
  return core::io::writeDeclaration(pStream, _T("scene::CSplineNode"), this, pSrv, 0, pszResolver);
}


//-------------------------------------------------------
// @name : CSplineNode::saveData
// Comment  
// @param core::io::IStream* pStream     : 
// @param DWORD dwID                     : 
// @param services::IOpenSaveService* pSrv : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineNode::saveData(core::io::IStream* pStream, DWORD dwID, services::IOpenSaveService* pSrv)
{
  //I force spline data save explicitly here, since I need it loaded before splineNode!
  ZPtr<core::io::ISerializeable>  pSerial;
  if (ZRESULT_OK == m_pSpline->QueryInterface(IID_(core::io::ISerializeable), (void**)&pSerial))
    pSerial->saveData(pStream, pSrv->getUnknownID(m_pSpline->GetControllingUnknown()), pSrv);

  core::io::CDataChunk chunk(pStream, dwID, 0x226);
  core::io::write(*(ZEntityBase*)this, pStream);
  core::io::write(static_cast<core::INamed*>(this), pStream);
  core::io::write(static_cast<scene::ITransformable*>(this), pStream);
  core::io::write(static_cast<scene::IBoundable*>(this), pStream);
  core::io::writeInterface(pStream, pSrv, m_pRenderTechnique->GetControllingUnknown());
  core::io::writeInterface(pStream, pSrv, m_pSpline->GetControllingUnknown());

  //user options is an embedded object, store it explicitly:
  if (m_pUserOptions)
    m_pUserOptions->saveData(pStream, /*id*/0, pSrv);

  return ZRESULT_OK;
}

ZRESULT CSplineNode::loadData(core::io::IStream* pStream, services::IOpenSaveService* pSrv, DWORD dwSize, DWORD dwVersion, bool bMerge)
{
  core::io::read(*(ZEntityBase*)this, pStream);
  core::io::read(static_cast<core::INamed*>(this), pStream);
  core::io::read(static_cast<scene::ITransformable*>(this), pStream);
  core::io::read(static_cast<scene::IBoundable*>(this), pStream);
  core::io::readInterface(pStream, pSrv, IID_(rend::IRenderTechnique), (void**)&m_pRenderTechnique);
  core::io::readInterface(pStream, pSrv, IID_(scene::ISpline), (void**)&m_pSpline);

  //
  // load user options, if available
  if (dwVersion >= 0x226)
  {
    pStream->read(&chunkInfo, sizeof(chunkInfo));//chunk of "m_pUserOptions"
    m_pUserOptions->loadData(pStream, pSrv, chunkInfo.m_dwSize, chunkInfo.m_dwVersion, bMerge);
  }

  // re-create manipulators:
  updateManipulators();
  return ZRESULT_OK;
}

ZRESULT CSplineNode::getReCreationMode(core::io::eReCreationMode& mode)
{
  mode = core::io::ERC_TRANSIENT_CLASS;
  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CSpline::saveDeclaration
// Saves declaration
// @param core::io::IStream* pStream     : 
// @param services::IOpenSaveService* pSrv : 
// @param LPCTSTR pszResolver            : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSpline::saveDeclaration(core::io::IStream* pStream, services::IOpenSaveService* pSrv, LPCTSTR pszResolver)
{
  return core::io::writeDeclaration(pStream, _T("scene::CSpline"), this, pSrv, 0, pszResolver);
}


//-------------------------------------------------------
// @name : CSpline::saveData
// Comment  
// @param core::io::IStream* pStream     : 
// @param DWORD dwID                     : 
// @param services::IOpenSaveService* pSrv : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSpline::saveData(core::io::IStream* pStream, DWORD dwID, services::IOpenSaveService* pSrv)
{
  core::io::CDataChunk chunk(pStream, dwID);
  pStream->write(&m_bClosed, sizeof(bool));
  //write sequence of control points;
  ZPtr<scene::IControlPoint>  ptr;
  long len = length();
  pStream->write(&len, sizeof(long));
  reset();
  while (ZRESULT_OK == next(&ptr))
  {
    eControlPoint type;
    ZPoint3D      pt;
    ZVector       v;
    long          nDetail;
    bool          bAuto;
    core::io::write(*static_cast<ZEntityBase*>(ptr), pStream);
    ptr->getType(type);
    pStream->write(&type, sizeof(type));
    ptr->getControlPosition(pt);
    pStream->write((float*)pt, sizeof(float)*3);
    ptr->getControlVector(scene::SCV_U0, v);
    pStream->write((float*)v, sizeof(float)*3);
    ptr->getControlVector(scene::SCV_U1, v);
    pStream->write((float*)v, sizeof(float)*3);
    ptr->getControlVector(scene::SCV_V0, v);
    pStream->write((float*)v, sizeof(float)*3);
    ptr->getControlVector(scene::SCV_V1, v);
    pStream->write((float*)v, sizeof(float)*3);
    ptr->getDetalization(nDetail, &bAuto);
    pStream->write(&nDetail, sizeof(nDetail));
    pStream->write(&bAuto, sizeof(bAuto));
    ptr = NULL;
  }
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CSpline::loadData
// Loads spline data
// @param core::io::IStream* pStream     : 
// @param services::IOpenSaveService* pSrv : 
// @param DWORD dwSize                   : 
// @param DWORD dwVersion                : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSpline::loadData(core::io::IStream* pStream, services::IOpenSaveService* pSrv, DWORD dwSize, DWORD dwVersion, bool bMerge)
{
  pStream->read(&m_bClosed, sizeof(bool));
  //write sequence of control points;
  ZPtr<scene::CControlPoint>  ptr;
  long len;
  clear();
  pStream->read(&len, sizeof(long));
  for (int i = 0; i < len; i++)
  {
    eControlPoint type;
    ZPoint3D      pt;
    ZVector       v;
    long          nDetail;
    bool          bAuto;
    scene::CControlPoint::CreateInstance(&ptr);
    core::io::read(*static_cast<ZEntityBase*>(ptr), pStream);
    pStream->read(&type, sizeof(type));
    ptr->setType(type);
    pStream->read((float*)pt, sizeof(float)*3);
    ptr->setControlPosition(pt);
    pStream->read((float*)v, sizeof(float)*3);
    ptr->setControlVector(scene::SCV_U0, v);
    pStream->read((float*)v, sizeof(float)*3);
    ptr->setControlVector(scene::SCV_U1, v);
    pStream->read((float*)v, sizeof(float)*3);
    ptr->setControlVector(scene::SCV_V0, v);
    pStream->read((float*)v, sizeof(float)*3);
    ptr->setControlVector(scene::SCV_V1, v);
    pStream->read(&nDetail, sizeof(nDetail));
    pStream->read(&bAuto, sizeof(bAuto));
    ptr->setDetalization(nDetail, bAuto);
    addLast(ptr);
    ptr = NULL;
  }
  return ZRESULT_OK;
}

ZRESULT CSpline::getReCreationMode(core::io::eReCreationMode& mode)
{
  mode = core::io::ERC_TRANSIENT_CLASS;
  return ZRESULT_OK;
}


};//scene


namespace rend
{

//-------------------------------------------------------
// @name : CSplineRenderTechnique::saveDeclaration
// Saves declaration
// @param core::io::IStream* pStream     : 
// @param services::IOpenSaveService* pSrv : 
// @param LPCTSTR pszResolver            : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineRenderTechnique::saveDeclaration(core::io::IStream* pStream, services::IOpenSaveService* pSrv, LPCTSTR pszResolver)
{
  return core::io::writeDeclaration(pStream, _T("rend::CSplineRenderTechnique"), this, pSrv, 0, pszResolver);
}

//-------------------------------------------------------
// @name : CSplineRenderTechnique::saveData
// Comment  
// @param core::io::IStream* pStream     : 
// @param DWORD dwID                     : 
// @param services::IOpenSaveService* pSrv : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CSplineRenderTechnique::saveData(core::io::IStream* pStream, DWORD dwID, services::IOpenSaveService* pSrv)
{
  core::io::CDataChunk chunk(pStream, dwID);
  core::io::write(static_cast<core::INamed*>(this), pStream);
  return ZRESULT_OK;
}

ZRESULT CSplineRenderTechnique::loadData(core::io::IStream* pStream, services::IOpenSaveService* pSrv, DWORD dwSize, DWORD dwVersion, bool bMerge)
{
  core::io::read(static_cast<core::INamed*>(this), pStream);
  return ZRESULT_OK;
}

ZRESULT CSplineRenderTechnique::getReCreationMode(core::io::eReCreationMode& mode)
{
  mode = core::io::ERC_TRANSIENT_CLASS;
  return ZRESULT_OK;
}


};//rend

